#ifndef PERL_UPB_T_C_CONVERT_TYPES_SFIXED64_H_
#define PERL_UPB_T_C_CONVERT_TYPES_SFIXED64_H_

#include "t/c/convert/test_util.h"
#include "t/c/upb-perl-test.h"

extern const upb_to_sv_test_case sfixed64_upb_to_sv_test_cases[];
extern const sv_to_upb_test_case sfixed64_sv_to_upb_test_cases[];

#endif /* PERL_UPB_T_C_CONVERT_TYPES_SFIXED64_H_ */
